@@ -14,7 +14,7 @@ BEGIN {
 }
 
 use vars qw( $VERSION );
-$VERSION = '1.10';
+$VERSION = '1.12';
 
 use constant ABBR_PARAM_SEPARATOR        => ', ';
 use constant ABBR_PARAM_SEPARATOR_LENGTH => length( ABBR_PARAM_SEPARATOR );
@@ -22,6 +22,7 @@ use constant DEPTH_STR                   => '  ';
 
 our $USAGE_HEADER = '';
 our $USAGE_FOOTER = '';
+our $SWITCH_PREFIX_STR = '-';
 
 =head1 NAME
 
@@ -144,15 +145,22 @@ L<print_usage>.
 
 =head3 parse_command_line
 
-Parses the command line based on the specified data structure.
+Parses the command line (@ARGV) based on the specified data structure.
 
-Accepts a single required parameter, an array reference describing the possible
-command line parameters. It returns two values, the "top level" option and a
-hashref of the other options specified.
+Accepts two parameters. The first is a required array reference describing the
+possible command line parameters. It returns three values, the "top level"
+option, a hashref of the other specified options, and an arrayref of any
+remaining unparsed options (similar to Getopt::Long).
 
-If the command line was unable to be parsed (the user specified an invalid
-option, the passed data structure was inconsistent, etc), parse_command_line
-will die with an appropriate error message.
+The second parameter is an optional array reference or string from which the
+parameters are to be read, rather than reading them from @ARGV. If a string is
+passed, it will be converted to an array via C<Text::ParseWords::shellwords>.
+
+If the command line was unable to be parsed (the passed data structure was
+inconsistent, etc), parse_command_line will die with an appropriate error
+message. If the command line was invalid (the user entered something that did
+not meet the given requirements, etc) a warning will be printed and undef will
+be returned.
 
 =head3 print_usage
 
@@ -160,7 +168,31 @@ Prints usage information based on the specified data structure.
 
 Takes two parameters, the first is a required array reference describing the
 possible command line parameters, and the second is an optional file handle to
-which the usage information will be printed.
+which the usage information will be printed. Alternately, in place of a
+filehandle, a hashref of options can be passed. Valid options are:
+
+=over 4
+
+=item fh
+
+The filehandle to print to. If not set, no output is printed.
+
+=item return
+
+A boolean value that determines whether or not the usage should be returned as
+a string.
+
+=item wrap_at
+
+Number of characters to wrap the output at. Will be auto-detected from
+$ENV{COLUMNS} if not set, defaults to 80 if auto-detection fails.
+
+=item hide_top_line
+
+Hides the top line of output which contains the program name how to pass
+switches.
+
+=back
 
 Usage information is generated mostly from the "descr" fields in the data
 structure, indentation is based on parameter dependence, parameters that accept
@@ -194,10 +226,6 @@ simply be described as the distance to the top of a tree, where as a "branch"
 would be the specific set of dependents for a given command, irrespective of
 dependents of commands on the same level.
 
-A top level command "-help" is always available and, if found, will call
-L<print_usage> on the current data structure and send the results to STDOUT and
-then exit(0).
-
 =head3 Data Structure
 
 The data structure is composed of an array of hashrefs. Each hashref describes
@@ -277,6 +305,12 @@ Text to be printed near the top of the "usage" output.
 
 Text to be printed at the end of the "usage" output.
 
+=head2 $Getopt::Tree::SWITCH_PREFIX_STR
+
+Characters to prefix a switch. Defaults to a single hyphen ('-'). Can be set to
+an empty string to use switchless options (Note: This option is not well
+tested!).
+
 =head1 NOTES
 
 You can't have a dependent parameter of the same name as a non-optional
@@ -307,7 +341,7 @@ sub process_getopt_params {
     foreach my $param_ref ( @{$param_array} ) {
         if ( !$param_ref->{name} ) { die "Parameter lacks a name!"; }
         if ( $param_ref->{name} =~ /[=|@]/ ) { die "Parameter names should not contain [=|@]!"; }
-        if ( $param_ref->{name} =~ /^-/ ) { die "Parameter names should not contain begin with a dash!"; }
+        if ( $param_ref->{name} =~ /^-/ ) { die "Parameter names should not begin with a dash!"; }
         if ( $param_ref->{abbr} ) {
             if ( $param_ref->{abbr} =~ /[-=|]@/ ) { die "Parameter abbreviations should not contain [=|@]!"; }
             if ( $param_ref->{abbr} =~ /^-/ ) { die "Parameter abbreviations should not begin with a dash!"; }
@@ -378,7 +412,7 @@ sub check_parameter {
     my @approved_flags;
     my $this_level;
     my $this_level_is_a_leaf = 1;
-    # This is a big of a pain, but we have to track existence of leaves
+    # This is a bit of a pain, but we have to track existence of leaves
     # separately from regular parameters. Since we don't recurse into a leaf
     # like we do a branch, we can't let check_parameter() handle the existence
     # test for us.
@@ -392,7 +426,7 @@ sub check_parameter {
             $this_level_is_a_leaf = 0;
         }
         # This level has a flag that was passed on the command line
-        if ( $g_opts->{ $top_level->{name} } ) {
+        if ( defined $g_opts->{ $top_level->{name} } ) {
             # We only get one non-optional command per branch in the tree
             if ( $top_level->{optional} ) {
                 my ( $status, $v ) = calc_param_value( $top_level, $g_opts );
@@ -449,33 +483,43 @@ sub check_parameter {
 # parameter we matched on) and the hash of parameter name => value pairs.
 # This function is exported.
 sub parse_command_line {
-    my ( $params ) = @_;
+    my ( $params, $source ) = @_;
     my @getopt_params;
     my %g_opts;
     my $status;
     my $op_ref;
     my $argv_index = 0;
     my $op;
+    my $remaining_options;
 
-    foreach ( @ARGV ) {
-        if ( $_ eq '-help' ) {
-            print_usage( $params );
-            exit 0;
-        }
+    if ( !$SWITCH_PREFIX_STR || $SWITCH_PREFIX_STR ne '-' ) {
+        my $t = $SWITCH_PREFIX_STR || '(?:)';
+        Getopt::Long::Configure( "prefix_pattern=$t" );
     }
 
-    # We can check ARGV for the first top level branch (non-optional and
-    # non-leaf) here, and then use that when we process that branch (only)
+    if ( !defined $source ) {
+        $source = \@ARGV;
+    } elsif ( !ref $source ) {
+        require Text::ParseWords;
+        $source = [ Text::ParseWords::shellwords($source) ];
+    }
+
+    # We can check our ARGV source for the first top level branch (non-optional
+    # and non-leaf) here, and then use that when we process that branch (only)
     # below.
     ARGV_INDEX_LOOP: while ( 1 ) {
-        $op = $ARGV[ $argv_index++ ];
+        $op = $source->[ $argv_index++ ];
         last if !defined $op;
 
         # If we encounter a value without a parameter before we find a top
         # level op, abort with an error. Any values tied to a parameter should
         # be skipped in the loop below.
-        die "The command '$op' is not valid!\n" unless $op =~ /^-/;
-        $op =~ s/^-//;
+        # Is this really necessary? Any problems would be caught below
+        #  --jeagle 20110411
+        #die "The command '$op' is not valid!\n" unless $op =~ /^-/;
+        if ( $SWITCH_PREFIX_STR ) {
+            $op =~ s/^\Q$SWITCH_PREFIX_STR\E//;
+        }
         foreach my $known_ops ( @{$params} ) {
             if ( !$known_ops->{name} ) { die 'Invalid name!'; }
             if (   ( $op eq $known_ops->{name} )
@@ -493,13 +537,13 @@ sub parse_command_line {
                 #            if ( !$known_ops->{params} ) { return $op, { $op => 1 }; }
             }
         }
-        if ( !$op_ref ) { die "The command '$op' is unknown!\n"; }
+        if ( !$op_ref ) {
+            warn "The command '$op' is unknown!\n";
+            return;
+        }
     }
 
-    if ( !$op ) {
-        print_usage( $params );
-        exit 1;
-    }
+    if ( !$op ) { return; }
 
     # Gather up all of the global non-branch parameters to pass to GetOptions.
     # While we're at it, check to make sure the parameters are correct
@@ -521,26 +565,26 @@ sub parse_command_line {
                 }
             }
         }
-        #if ( !exists $g_opts{$p} ) { $g_opts{$p} = 0; }
-        #$g_opts{$p} = 0;
     }
-    #@getopt_params = keys( %g_opts );
     undef %g_opts;
+
     eval {
         local $SIG{__WARN__} = sub { };
-        GetOptions( \%g_opts, @getopt_params );
+        # GetOptionsFromArray behaves badly unless called fully qualified. Not
+        # interested in tracking down why.
+        ( my $getopt_success, $remaining_options ) =
+            Getopt::Long::GetOptionsFromArray( $source, \%g_opts, @getopt_params );
+        return unless $getopt_success;
     };
     undef @getopt_params;
 
     # Find the top-level parameter we're going to work with.
     ( $status, @getopt_params ) = check_parameter( \%g_opts, [$op_ref] );
-    #warn "Check parameter failed!\n\n" unless $status;
-    return                             unless $status;
+    return unless $status;
 
     if ( @global_options ) {
         ( $status, @global_options ) = check_parameter( \%g_opts, \@global_options );
-        #warn "Check global parameter failed!\n" unless $status;
-        return                                  unless $status;
+        return unless $status;
         push @getopt_params, @global_options;
     }
 
@@ -561,7 +605,7 @@ sub parse_command_line {
         warn "Not all good!";
         return;
     }
-    return $op, \%g_opts;
+    return $op, \%g_opts, $remaining_options;
 }
 
 # Pre-scan to get the width of all of the parameters
@@ -597,6 +641,7 @@ sub print_actual_usage {
     if ( !defined $depth ) { $depth = 0; }
     my $d   = DEPTH_STR x $depth;
     my $out = '';
+    my $switch_prefix = $SWITCH_PREFIX_STR || '';
     return '' unless ref $params eq 'ARRAY';
 
     # Print the parameters (recursively) with optional parameters first.
@@ -614,11 +659,11 @@ sub print_actual_usage {
     {
         # Don't show the leading dash on the first parameter (command).
         #my $param_desc = ( ( $depth == 0 && !$top_level->{optional} ) ? '' : '-' ) . $top_level->{name};
-        my $param_desc = '-' . $top_level->{name};
+        my $param_desc = $switch_prefix . $top_level->{name};
         if ( $top_level->{abbr} ) {
             $param_desc = $param_desc . ABBR_PARAM_SEPARATOR
              #. ( ( $depth == 0 && !$top_level->{optional} ) ? '' : '-' )
-             . '-' . $top_level->{abbr};
+             . $switch_prefix . $top_level->{abbr};
         }
         if ( !$top_level->{exists} ) {
             $param_desc .= ' <..>';
@@ -640,17 +685,39 @@ sub print_actual_usage {
     return $out;
 }
 
-# Calculate widths and set off the recursive call to print_actual_usage
+# Calculate widths and set off the recursive call to print_actual_usage.
 # This function is exported.
+# Woah! Why did this get so complicated? Well, prior to 1.12 we accepted the
+# second parameter as a filehandle to print to. In 1.12 I realized that was
+# silly, and it would be better to just return the usage as a string. So now,
+# for backward-compatability sake, we must support all of the options. Passing
+# a filehandle causes us to print to that with an appropriate width. No
+# filehandle will assume STDOUT as the filehandle. Passing a hash causes us to
+# read various options from that hash.
 sub print_usage {
-    my ( $params, $fh )  = @_;
-    my $param_width   = get_usage_param_width( $params );
-    my $wrap_at       = $ENV{COLUMNS};
-    if ( !$fh ) { $fh = *STDERR; }
-    if ( ( !defined $wrap_at ) || ( $wrap_at !~ /^\d{1,4}$/ ) || ( !-t STDERR ) ) { $wrap_at = 80; }
+    my ( $params, $fh_or_options )  = @_;
+    my $param_width = get_usage_param_width( $params );
+    my $options;
+
+    if ( !defined $fh_or_options ) {
+        $options = { fh => *STDERR, return => 0 };
+    } elsif ( ref $fh_or_options eq 'HASH' ) {
+        # Copy it in case the caller wants to re-use.
+        $options = \%{ $fh_or_options };
+    } else {
+        $options = { fh => $fh_or_options, return => 0 };
+    }
+    if ( $options->{fh} && !$options->{wrap_at} ) {
+        if ( ( -t $options->{fh} ) && ( defined $ENV{COLUMNS} ) ) {
+            $options->{wrap_at} = $ENV{COLUMNS};
+        }
+    }
+    if ( !$options->{wrap_at} || $options->{wrap_at} !~ /^\d{1,4}$/ ) {
+        $options->{wrap_at} = 80;
+    }
 
     local $Text::Wrap::unexpand = 0;
-    local $Text::Wrap::columns  = $wrap_at;
+    local $Text::Wrap::columns  = $options->{wrap_at};
     my $usage_short = '';
 
     foreach my $p ( @{ $params } ) {
@@ -659,18 +726,38 @@ sub print_usage {
         }
     }
 
-    print $fh 'Usage: ',
-     basename( $0 ),
-     wrap( '', '', " <-help|command> ${usage_short}\[flags\]\n" ),
-     "\n",
-     $USAGE_HEADER,
-     "Options:\n",
-     print_actual_usage( $params, $param_width ), "\n",
-     $USAGE_FOOTER;
+    my $usage_str = '';
+
+    if ( !$options->{hide_top_line} ) {
+        $usage_str = 'Usage: ' .  basename( $0 ) .  wrap( '', '', " <command> ${usage_short}\[flags\]\n" ) . "\n";
+    }
+
+    if ( $USAGE_HEADER ) {
+        $usage_str .= $USAGE_HEADER;
+    }
+
+    $usage_str .= "Options:\n" . print_actual_usage( $params, $param_width ), "\n";
+
+    if ( $USAGE_FOOTER ) {
+        $usage_str .= $USAGE_FOOTER;
+    }
+
+    if ( $options->{fh} ) { print {$options->{fh}} $usage_str; }
+    if ( $options->{return} ) { return $usage_str; }
 }
 
 =head1 CHANGES
 
+=head2 Version 1.12, 20100411, jeagle
+
+Add ability to return usage as a string.
+
+Add ability to set the prefix character via $Getopt::Tree::SWITCH_PREFIX_STR
+
+Remove automatic -help flag.
+
+Fix a silly bug causing parameter values that evaluate to false to fail.
+
 =head2 Version 1.11, 20100917, jeagle
 
 Appease older versions of Perl in print_usage's usage of square brackets in
